package com.bonus.material.basic.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import com.bonus.common.log.enums.OperaType;
import com.bonus.material.common.annotation.PreventRepeatSubmit;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.bonus.common.log.annotation.SysLog;
import com.bonus.common.security.annotation.RequiresPermissions;
import com.bonus.material.basic.domain.BmAgreementInfo;
import com.bonus.material.basic.service.IBmAgreementInfoService;
import com.bonus.common.core.web.controller.BaseController;
import com.bonus.common.core.web.domain.AjaxResult;
import com.bonus.common.core.utils.poi.ExcelUtil;
import com.bonus.common.core.web.page.TableDataInfo;

/**
 * 协议管理Controller
 * 
 * @author xsheng
 * @date 2024-09-27
 */
@Api(tags = "协议管理接口")
@RestController
@RequestMapping("/bm_agreement_info")
public class BmAgreementInfoController extends BaseController
{
    @Autowired
    private IBmAgreementInfoService bmAgreementInfoService;

    /**
     * 查询协议管理列表
     */
    @ApiOperation(value = "查询协议管理列表")
    //@RequiresPermissions("basic:info:list")
    @GetMapping("/list")
    public TableDataInfo list(BmAgreementInfo bmAgreementInfo)
    {
        startPage();
        List<BmAgreementInfo> list = bmAgreementInfoService.selectBmAgreementInfoList(bmAgreementInfo);
        return getDataTable(list);
    }

    /**
     * 导出协议管理列表
     */
    @ApiOperation(value = "导出协议管理列表")
    @PreventRepeatSubmit
    //@RequiresPermissions("basic:info:export")
    @SysLog(title = "协议管理", businessType = OperaType.EXPORT, logType = 1,module = "仓储管理->导出协议管理")
    @PostMapping("/export")
    public void export(HttpServletResponse response, BmAgreementInfo bmAgreementInfo)
    {
        List<BmAgreementInfo> list = bmAgreementInfoService.selectBmAgreementInfoList(bmAgreementInfo);
        ExcelUtil<BmAgreementInfo> util = new ExcelUtil<BmAgreementInfo>(BmAgreementInfo.class);
        util.exportExcel(response, list, "协议管理数据");
    }

    /**
     * 获取协议管理详细信息
     */
    @ApiOperation(value = "获取协议管理详细信息")
    //@RequiresPermissions("basic:info:query")
    @GetMapping(value = "/{agreementId}")
    public AjaxResult getInfo(@PathVariable("agreementId") Long agreementId)
    {
        return success(bmAgreementInfoService.selectBmAgreementInfoByAgreementId(agreementId));
    }

    /**
     * 新增协议管理
     */
    @ApiOperation(value = "新增协议管理")
    @PreventRepeatSubmit
    //@RequiresPermissions("basic:info:add")
    @SysLog(title = "协议管理", businessType = OperaType.INSERT, logType = 1,module = "仓储管理->新增协议管理")
    @PostMapping
    public AjaxResult add(@RequestBody BmAgreementInfo bmAgreementInfo)
    {
        try {
            return bmAgreementInfoService.insertBmAgreementInfo(bmAgreementInfo);
        } catch (Exception e) {
            return error("系统错误, " + e.getMessage());
        }
    }

    /**
     * 修改协议管理
     */
    @ApiOperation(value = "修改协议管理")
    @PreventRepeatSubmit
    //@RequiresPermissions("basic:info:edit")
    @SysLog(title = "协议管理", businessType = OperaType.UPDATE, logType = 1,module = "仓储管理->修改协议管理")
    @PutMapping
    public AjaxResult edit(@RequestBody BmAgreementInfo bmAgreementInfo)
    {
        return bmAgreementInfoService.updateBmAgreementInfo(bmAgreementInfo);
    }

    /**
     * 删除协议管理
     */
    @ApiOperation(value = "删除协议管理")
    @PreventRepeatSubmit
    //@RequiresPermissions("basic:info:remove")
    @SysLog(title = "协议管理", businessType = OperaType.DELETE, logType = 1,module = "仓储管理->删除协议管理")
	@DeleteMapping("/{agreementId}")
    public AjaxResult remove(@PathVariable("agreementId")  Long agreementId)
    {
        return toAjax(bmAgreementInfoService.deleteBmAgreementInfoByAgreementId(agreementId));
    }
}
